<?php

declare(strict_types=1);

namespace Drupal\oktaplus\EventSubscriber;

use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Routing\RouteBuilderInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

final class ConfigEventSubscriber implements EventSubscriberInterface {

  public function __construct(
    protected RouteBuilderInterface $routeBuilder,
  ) {}

  /**
   * Rebuild router if path changed.
   */
  public function onConfigSave(ConfigCrudEvent $event): void {
    if ($event->getConfig()->getName() === 'oktaplus.settings' && $event->isChanged('event_hook_path')) {
      $this->routeBuilder->rebuild();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[ConfigEvents::SAVE][] = ['onConfigSave'];
    return $events;
  }

}
