<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\OktaLogEvent;
use Drupal\oktaplus\OktaplusUserUtility;
use Drupal\user\UserInterface;

final class UserUtilityTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'oktaplus',
    'openid_connect',
    'externalauth',
    'user',
    'file',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installSchema('externalauth', ['authmap']);
    $this->installSchema('user', ['users_data']);
    $this->container->get('config.factory')
      ->getEditable('oktaplus.settings')
      ->set('client', 'okta')
      ->save();
  }

  /**
   * @covers \Drupal\oktaplus\OktaplusUserUtility::createUserFromEvent
   */
  public function testCreateUserFromEvent(): void {
    $email = $this->randomMachineName() . '@example.com';
    $utility = \Drupal::service(OktaplusUserUtility::class);
    $event = OktaLogEvent::fromArray([
      'uuid' => $this->randomMachineName(),
      'eventType' => EventType::ApplicationUserMembershipAdd->value,
      'published' => '2024-07-30T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
      'target' => [
        [
          'id' => $this->randomMachineName(),
          'type' => 'User',
          'alternateId' => $email,
          'displayName' => $this->randomMachineName(),
        ],
      ],
    ]);
    $user = $utility->createUserFromEvent($event);
    $this->assertInstanceOf(UserInterface::class, $user);
    $this->assertSame($email, $user->getEmail());
    $this->assertSame($email, $user->getAccountName());
  }

  /**
   * @covers \Drupal\oktaplus\OktaplusUserUtility::activateUser
   */
  public function testActivateUser(): void {
    $user = $this->createMock(UserInterface::class);
    $user->expects($this->once())
      ->method('activate')
      ->willReturnSelf();
    $logEvent = OktaLogEvent::fromArray([
      'uuid' => $this->randomMachineName(),
      'eventType' => EventType::ApplicationUserMembershipAdd->value,
      'published' => '2024-07-30T00:00:00.000Z',
      'outcome' => [
        'result' => EventOutcome::Success->value,
      ],
    ]);
    $utility = \Drupal::service(OktaplusUserUtility::class);
    $utility->activateUser($user, $logEvent);
  }

}
