<?php

declare(strict_types=1);

namespace Drupal\Tests\oktaplus\Unit;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\externalauth\AuthmapInterface;
use Drupal\oktaplus\Enum\EventOutcome;
use Drupal\oktaplus\Enum\EventType;
use Drupal\oktaplus\OktaLogEvent;
use Drupal\oktaplus\OktaplusUserUtility;
use Drupal\Tests\UnitTestCase;
use Drupal\user\UserInterface;
use Drupal\user\UserStorage;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * @coversDefaultClass \Drupal\oktaplus\OktaplusUserUtility
 */
final class UserUtilityTest extends UnitTestCase {

  /**
   * @covers ::getTargetUser
   */
  public function testGetTargetUser(): void {
    $drupalUserId = 1000;
    $oktaUserId = $this->randomMachineName();

    $config = $this->createMock(Config::class);
    $config->expects($this->once())
      ->method('get')
      ->with('client')
      ->willReturn('test_client');

    $configFactory = $this->createMock(ConfigFactoryInterface::class);
    $configFactory->expects($this->once())
      ->method('get')
      ->with('oktaplus.settings')
      ->willReturn($config);

    $user = $this->createMock(UserInterface::class);
    $user->expects($this->once())
      ->method('id')
      ->willReturn($drupalUserId);

    $userStorage = $this->createMock(UserStorage::class);
    $userStorage->expects($this->once())
      ->method('load')
      ->with($drupalUserId)
      ->willReturn($user);

    $entityTypeManager = $this->createMock(EntityTypeManagerInterface::class);
    $entityTypeManager->expects($this->once())
      ->method('getStorage')
      ->with('user')
      ->willReturn($userStorage);

    $authmap = $this->createMock(AuthmapInterface::class);
    $authmap->expects($this->once())
      ->method('getUid')
      ->with($oktaUserId, 'openid_connect.test_client')
      ->willReturn($drupalUserId);

    $event = new OktaLogEvent(
      $this->randomMachineName(),
      new \DateTimeImmutable(),
      EventType::UserLifecycleDeactivate,
      EventOutcome::Success,
      [
        'target' => [
          [
            'type' => 'User',
            'id' => $oktaUserId,
          ],
        ],
      ],
    );

    $utility = new OktaplusUserUtility(
      $configFactory,
      $entityTypeManager,
      $authmap,
      $this->createMock(EventDispatcherInterface::class),
    );

    $user = $utility->getTargetUser($event);
    $this->assertSame($drupalUserId, $user?->id());
  }

}
